TOOL_NAME = "Toggle Master Meter Ed."

renoise.tool():add_keybinding {
  name = "Global:Tools:`TMT` Toggle Master Plug Ed. 1", 
  invoke = function()toggle_plug(1) end  
}

renoise.tool():add_keybinding {
  name = "Global:Tools:`TMT` Toggle Master Plug Ed. 2", 
  invoke = function()toggle_plug(2) end  
}

renoise.tool():add_keybinding {
  name = "Global:Tools:`TMT` Toggle Master Plug Ed. 3", 
  invoke = function()toggle_plug(3) end  
}

renoise.tool():add_keybinding {
  name = "Global:Tools:`TMT` Toggle Master Plug Prefs", 
  invoke = function()prefs_gui() end  
}

--------------------------------------------------------------------------------
--some basic colors for gui elements
--------------------------------------------------------------------------------
--e.g. For changing vb.views["sample present colour 2"].color when states change
COLOR_GREY = {0x30,0x42,0x42}
COLOR_GREY_MILD = {0x50,0x74,0x74}
COLOR_ORANGE ={0xFF,0x66,0x00}
COLOR_YELLOW = {0xE0,0xE0,0x00}
COLOR_BLUE = {0x50,0x40,0xE0}  
COLOR_RED = {0xEE,0x10,0x10}
COLOR_GREEN = {0x20,0x99,0x20}
COLOR_RED_MILD = {0x90,0x10,0x10}

--Constants holding first index of color-code: Used to identify color in vb.views table i.e.
--if vb.views["button"].color[1] == COLOR_ORANGE_FLAG then --etc
COLOR_GREY_FLAG = COLOR_GREY[1]
COLOR_GREY_MILD_FLAG = COLOR_GREY_MILD[1]
COLOR_ORANGE_FLAG = COLOR_ORANGE[1]
COLOR_YELLOW_FLAG = COLOR_YELLOW[1]
COLOR_BLUE_FLAG = COLOR_BLUE[1]  
COLOR_RED_FLAG = COLOR_RED[1]
COLOR_GREEN_FLAG = COLOR_GREEN[1]
COLOR_RED_MILD_FLAG = COLOR_RED_MILD[1]


LUA_COUNTS_FROM_1 = 1


------------------------------------------
--Set up Preferences file
------------------------------------------
--create xml
local options = renoise.Document.create {
  plug_1_path = "",
  plug_1_short_name = "",
  plug_2_path = "",
  plug_2_short_name = "",
  plug_3_path = "",
  plug_3_short_name = "",
}
 --assign options-object to .preferences so renoise knows to load and update it with the tool
renoise.tool().preferences = options
------------------------------------------
------------------------------------------
--variable syntax 
--options.value_a.value


------------------------------
function toggle_plug(shortcut)
------------------------------

  local plug_path = nil
  
  if shortcut == 1 then
    plug_path = options.plug_1_path.value
  elseif shortcut == 2 then
    plug_path = options.plug_2_path.value
  elseif shortcut == 3 then
    plug_path = options.plug_3_path.value
  end 
  
  --if path not set for this shortcut then open the prefs gui
  if plug_path == "" then
    prefs_gui(shortcut)
    return
  end
  

  local song = renoise.song()
  --get master track
  local mst_trk = song:track(song.sequencer_track_count + 1)
  
  --loop through track devices
  for device = 2,#mst_trk.devices do
    local dev = mst_trk:device(device)
    if dev.device_path == plug_path then
      
     
     --OPTION
     --[[move up to end of chain, -- done one device at a time with devices swap
      local second_last_pos = (#song.tracks[track].devices - 1)
      
      for pos = device,second_last_pos do
        song.tracks[track]:swap_devices_at(pos, pos+1)
        device = (pos + 1)--for opening editor
      end 
      --]]
      
      --toggle editor visible
      if dev.external_editor_available == true then
        dev.external_editor_visible = not dev.external_editor_visible
        return
      end
    end
  end
  --add gliss at end of chain as it was not found
  local new_dev = mst_trk:insert_device_at(plug_path, #mst_trk.devices + 1)
  --open the editor
  new_dev.external_editor_visible = true

end
---


--global variables for gui
local my_dialog = nil 
local vb = nil


--toggle the tool open and closed (keyboard shortcut start-up)
-------------------------------------------------------------
function main_toggle()
----------------------
 --close dialog if it is open
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    --reset global my_dialog
     my_dialog = nil 
  else --run main
    main()
  end
end

--always open/ restart tool (menu entry start-up)
-------------------------------------------------
function main_start()
---------------------
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    --reset global my_dialog
     my_dialog = nil 
  end
  --run main
  main()
end
----------------------------------------------------------


--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
 function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end

--https://forum.renoise.com/t/renoise-3-1-1-help-with-print-showing-in-terminal/49800/5
------------------
function dbug(msg)
------------------
  local base_types = {
    ["nil"] = true, ["boolean"] = true, ["number"] = true,
    ["string"] = true, ["thread"] = true, ["table"] = true
  }
  if not base_types[type(msg)] then oprint(msg)
  elseif type(msg) == 'table' then rprint(msg)
  else print(msg) end
end


------------------------------------
------------------------------------
function prefs_gui(shortcut)
------------------------------------
------------------------------------


  --toggle dialog so if its already open then close it.  The timer will catch
  --that it is closed and do the housekeeping like removing notifiers (and timer itself)
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    return
  end
  
  --renoise song object
  local song = renoise.song()

  --set globals
  -------------
  --assign global vb to viewbuilder
  vb = renoise.ViewBuilder()
  
  ----------------------------------------------
  --GUI
  ----------------------------------------------     


  --dialog content will contain all of gui; passed to renoise.app():show_custom_dialog()
  local dialog_content = vb:column{
  
  margin = 4,
  vb:text{
   text = "Target Device",
  },
  vb:row{
   style = "border",
   vb:text{
    id = "target",
    width = 200,
    style = "strong",
    text = song.selected_device and song.selected_device.short_name or "Select A Device"
   
   }
  },
  
  vb:text{ --spacer
  },
  
  vb:row{
   margin = 2,
   vb:button{
     id = "button 1",
     color = COLOR_GREY,
     text = "Set Shortcut 1",
     notifier = function()
                  local song = renoise.song()
                  if song.selected_track_device ~= nil and song.selected_track_device_index ~= 1 then
                    options.plug_1_short_name.value = song.selected_track_device.short_name
                    options.plug_1_path.value = song.selected_track_device.device_path
                    vb.views["button 1"].color = COLOR_GREY
                  else
                    status("Needs To Be A Valid Device Selected")
                  end
                end,
   },
   vb:row{
    style = "border",
    vb:text{
     width = 114,
     style = "strong",
     id = "assign 1",
     text = options.plug_1_short_name.value,
     },
   },
    vb:button{
     text = "X",
     notifier = function()--clear plug choice from options
                  options.plug_1_short_name.value = ""
                  options.plug_1_path.value = ""
                end
    
    },
  },
  
  vb:row{
   margin = 2,
   vb:button{
     id = "button 2",
     color = COLOR_GREY,
     text = "Set Shortcut 2",
     notifier = function()--set plug options
                local song = renoise.song()
                if song.selected_track_device ~= nil and song.selected_track_device_index ~= 1 then
                  options.plug_2_short_name.value = song.selected_track_device.short_name
                  options.plug_2_path.value = song.selected_track_device.device_path
                  vb.views["button 2"].color = COLOR_GREY
                else
                  status("Needs To Be A Valid Device Selected")
                end
              end,
   },
   vb:row{
    style = "border",
    vb:text{
     width = 114,
     style = "strong",
     id = "assign 2",
     text = options.plug_2_short_name.value,
    },
   },
      vb:button{
     text = "X",
     notifier = function() --clear plug choice from options
                  options.plug_2_short_name.value = ""
                  options.plug_2_path.value = ""
                 end
    
    },
  },
  vb:row{
   margin = 2,
   vb:button{
     id = "button 3",
     color = COLOR_GREY,
     text = "Set Shortcut 3",
     notifier = function() --set plug options
                  local song = renoise.song()
                  if song.selected_track_device ~= nil and song.selected_track_device_index ~= 1 then
                    options.plug_3_short_name.value = song.selected_track_device.short_name
                    options.plug_3_path.value = song.selected_track_device.device_path
                    vb.views["button 3"].color = COLOR_GREY
                  else
                    status("Needs To Be A Valid Device Selected")
                  end
                end,
   },
  
  vb:row{
    style = "border",
    vb:text{
     width = 114,
     style = "strong",
     id = "assign 3",
     text = options.plug_3_short_name.value,
    },
   },
      vb:button{
     text = "X",
     notifier = function()--clear plug choice from options
                  options.plug_3_short_name.value = ""
                  options.plug_3_path.value = ""
                 end
    
    },
  },
 }
  
  
  
  
  
  
  



  
  --------------
  --key Handler
  --------------
  local function my_keyhandler_func(dialog,key)
     --toggle lock focus hack, allows pattern ed to get key input
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
    -- rprint(key)
     
     return key
     
  end 
  
 
  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      TOOL_NAME, dialog_content,my_keyhandler_func)

 
  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(timer) then
    renoise.tool():add_timer(timer,80)
  end

  
  --------------------------------------------    
  --close dialog function ON NEW SONG
  --------------------------------------------
  local function closer(d)
    --close dialog if exists and is open     
    if (d ~= nil) and (d.visible == true) then
      d:close()
     -- remove_notifiers()
    end
    --reset global my_dialog
     my_dialog = nil 
  end
  -- notifier to close dialog on load new song
  renoise.tool().app_release_document_observable:add_notifier(closer,my_dialog)
  -------------------------------------------------------------------------------
  print(shortcut)
  --when called by shortcut, hilight the shortcut button that called it
   if shortcut ~= nil then
    if shortcut == 1 then
      print("1")
      vb.views["button 1"].color = COLOR_YELLOW
    elseif shortcut == 2 then
      print("2")
      vb.views["button 2"].color = COLOR_YELLOW
    elseif shortcut == 3 then
      print("3")
      vb.views["button 3"].color = COLOR_YELLOW
    end
  end
  
  --first run of timer
  timer()
end


----------------
--timer function
----------------
function timer()
  
  --remove timer when GUI is closed
  if(my_dialog == nil) or (my_dialog.visible == false) then
    if renoise.tool():has_timer(timer) then
      renoise.tool():remove_timer(timer)
    end
  end
  
  local song = renoise.song()
 
  vb.views["assign 1"].text = " "..options.plug_1_short_name.value
  vb.views["assign 2"].text = " "..options.plug_2_short_name.value
  vb.views["assign 3"].text = " "..options.plug_3_short_name.value
  
  vb.views["target"].text = song.selected_track_device and song.selected_track_device.short_name or "                  Select A Device"
   

end


--[[ --bring dialog to the front
if (my_dialog and my_dialog.visible) then -- only allows one dialog instance
  my_dialog:show() --brings gui to front
  return
end
--]]


